@@ -1,3 +1,4 @@
+from copy import deepcopy
 from dataclasses import dataclass
 from typing import Optional, Union, NamedTuple
 import numpy as np
@@ -10,7 +11,7 @@ from ..paper.annotations import (
     tag_parsed_annotations,
 )
 from ..geometry import geom_ops
-from ..loads import load_distribution as ld
+import load_distribution as ld
 import parse
 import math
 import tomli_w
@@ -178,13 +179,16 @@ class Element:
         annotations_w_intersect_corrs = get_geometry_correspondents(
             annotations_w_intersect
         )
+        if correspond_with_like_only:
+            filtered_annots = filter_correspondents(annotations_w_intersect_corrs)
+        else:
+            filtered_annots = annotations_w_intersect_corrs
         trib_area_geoms = np.array(
             [annot_attrs["geometry"] for annot_attrs in trib_annotations.values()]
         )
         matching_trib_poly = None
         elements = []
-        for annot_attrs in annotations_w_intersect_corrs.values():
-            element_family = annot_attrs["tag"][0]
+        for annot_attrs in filtered_annots.values():
             geometry = annot_attrs["geometry"]
             if geometry.geom_type == "LineString" and trib_annotations:
                 intersection_mask = geometry.intersects(trib_area_geoms)
@@ -194,13 +198,6 @@ class Element:
                     get_intersection_lengths
                 )  # The matching trib is the one that is mostly in the trib poly
                 matching_trib_poly = trib_area_geoms[matching_trib_index]
-            if correspond_with_like_only:
-                corrs_a = prioritize_correspondents(
-                    annot_attrs["correspondents_above"], element_family
-                )
-                corrs_b = prioritize_correspondents(
-                    annot_attrs["correspondents_below"], element_family
-                )
 
             available_kwargs = {
                 k: v for k, v in annot_attrs.items() if k not in ELEMENT_ATTRS
@@ -212,8 +209,8 @@ class Element:
                 rank=annot_attrs["rank"],
                 intersections_above=annot_attrs["intersections_above"],
                 intersections_below=annot_attrs["intersections_below"],
-                correspondents_above=corrs_a,
-                correspondents_below=corrs_b,
+                correspondents_above=annot_attrs["correspondents_above"],
+                correspondents_below=annot_attrs["correspondents_below"],
                 plane_id=annot_attrs.get("page_label", None),
                 reaction_type=annot_attrs.get("reaction_type", "point"),
                 trib_area=matching_trib_poly,
@@ -268,23 +265,23 @@ class Element:
                 ]: ib.other_tag
                 for ib in self.intersections_below
             }
-            poly_support_geoms = list(support_tags_by_geom.keys())
-            support_geoms = geom_ops.clean_polygon_supports(
-                poly_support_geoms, self.geometry
-            )
-
+            support_geoms = list(support_tags_by_geom.keys())
             cleaned_supports_map = {}
-            for idx, poly_support_geom in enumerate(poly_support_geoms):
+            for idx, poly_support_geom in enumerate(support_geoms):
                 clean_support_geom = support_geoms[idx]
                 cleaned_supports_map.update({clean_support_geom: poly_support_geom})
-            ordered_support_geoms = geom_ops.sort_supports(self.geometry, support_geoms)
             try:
+                ordered_support_geoms = geom_ops.sort_supports(
+                    self.geometry, support_geoms
+                )
                 extents = geom_ops.get_joist_extents(
                     self.geometry,
                     ordered_support_geoms,
                     self.trib_area,
+                    extent_polygon=self.extent_polygon,
                 )
             except AssertionError as e:
+                # raise e
                 raise AssertionError(
                     f"No intersection within joist extents: {self.tag=}"
                 )
@@ -432,7 +429,7 @@ def prioritize_correspondents(
     for correspondent in correspondents:
         corr_family = correspondent.other_tag[0]
         corr_rank = correspondent.other_rank
-        if corr_rank == 0:  # If the element transfers out
+        if corr_rank == 0 and corr_family == family:  # If the element transfers out
             filtered_transfers.append(correspondent)
         elif corr_family == family:
             filtered_same_family.append(correspondent)
@@ -547,6 +544,8 @@ class LoadedElement(Element):
                 "horz_intersects_below": [
                     inter.other_tag for inter in self.intersections_below
                 ],
+                "reaction_type": self.reaction_type,
+                "rank": self.rank,
                 "user_defined": available_kwargs,
             },
             "element_geometry": {
@@ -594,7 +593,7 @@ class LoadedElement(Element):
                 supports_acc.append(
                     {"location": round(support_location, precision), "fixity": fixity}
                 )
-            return supports_acc
+            return sorted(supports_acc, key=lambda x: x["location"])
         else:
             return []
 
@@ -627,6 +626,17 @@ class LoadedElement(Element):
                     for intersection in self.intersections_above
                 ],
             )
+        elif self.geometry.geom_type == "Polygon" and self.reaction_type == "linear":
+            coords_a, coords_b = Point(coords_a), Point(coords_b)
+            ordered_coords = geom_ops.order_nodes_positive([coords_a, coords_b])
+            start_coord = ordered_coords[0]
+            transfer_locations = geom_ops.get_local_intersection_ordinates(
+                start_coord,
+                [
+                    intersection.intersecting_region
+                    for intersection in self.intersections_above
+                ],
+            )
         else:  # But not when it is a column
             start_coord = coords_a
             # This is where teh eccentricity can be calculated based on using the
@@ -814,16 +824,36 @@ class LoadedElement(Element):
                     area_dist_load = geom_ops.trapezoid_area(
                         h=(end_x - start_x), b1=start_y, b2=end_y
                     )
-                    trapezoid_ratio = area_dist_load / total_polygon_area
+                    try:
+                        if self.reaction_type == "point":
+                            # collectors from the JoistArrayModel have trib areas that reflect
+                            # the actual trib area for the subelement. Thus, the area reatio
+                            # should reflect the percentage of area from each distributed load
+                            # sub-section and what percentage that reflects of the complete
+                            # distributed load system which may consist of several trapezoids
+                            area_ratio = area_dist_load / total_polygon_area
+                        # THIS GIVES THE CORRECT TRAPEZOID RATIO FOR COLLECTORTRIBMODEL
+                        elif self.reaction_type == "linear":
+                            # In the CollectorTribModel the trib area reflects the size of a whole
+                            # spread of joists which will be reduced down to a reaction over a unit length.
+                            # In this case, we need an area ratio to reflect the percentage of area that
+                            # an area load covers in relation to the total trib area.
+                            area_ratio = (
+                                self.applied_loading_areas[idx][0].area
+                                / self.trib_area.area
+                            )
+                    except ZeroDivisionError:
+                        continue  # Skip this dist load if there is no polygon area
                     intersected_poly, applied_loading = self.applied_loading_areas[idx]
+                    if area_ratio == 0.0 and intersected_poly.area == 0.0:
+                        continue  # Skip this dist load if there is no intersection area
                     dist_load = {
                         "transfer_source": "",
                         "transfer_reaction_index": "",
                         "occupancy": applied_loading.occupancy,
                         "load_components": applied_loading.load_components or [],
-                        "applied_area": round(
-                            intersected_poly.area * trapezoid_ratio, precision
-                        ),
+                        "applied_area": round(intersected_poly.area, precision),
+                        "applied_area_ratio": area_ratio,
                         "start_loc": round(start_x, precision),
                         "start_magnitude": round(start_y, precision),
                         "end_loc": round(end_x, precision),
@@ -1021,13 +1051,19 @@ def get_geometry_intersections(
         intersections_below = []
         for j_annot in annots:
             j_attrs = intersected_annotations[j_annot]
-            j_rank = j_attrs["rank"]
+            try:
+                j_rank = j_attrs["rank"]
+            except KeyError:
+                print(j_annot, j_attrs)
+                raise ValueError
             j_page = j_annot.page
             i_geom = i_attrs["geometry"]
             j_geom = j_attrs["geometry"]
             i_tag = i_attrs["tag"]
             j_tag = j_attrs["tag"]
             j_extent_poly = j_attrs["extent_polygon"]
+            if "WT0.0" in (i_tag, j_tag):
+                print(i_tag, j_tag)
             if i_page != j_page:
                 continue
             if j_rank > i_rank:  # When i transfers to j
@@ -1050,6 +1086,8 @@ def get_geometry_intersections(
                     extent_intersection = True
                 else:
                     intersection = geom_ops.get_intersection(i_geom, j_geom, j_tag)
+                    if "WT0.0" in (j_tag, i_tag):
+                        print(intersection, i_geom, j_geom, i_tag, j_tag)
 
                 if intersection is None:
                     continue
@@ -1084,7 +1122,7 @@ def get_geometry_intersections(
                     )
                 ):
                     intersection = geom_ops.get_intersection(
-                        j_geom, i_geom, i_tag, j_extent_poly
+                        j_geom, i_geom, j_tag, j_extent_poly
                     )
                 else:
                     intersection = geom_ops.get_intersection(i_geom, j_geom, j_tag)
@@ -1094,7 +1132,7 @@ def get_geometry_intersections(
 
                 intersections_above.append(
                     Intersection(
-                        *intersection, other_reaction_type=i_attrs["reaction_type"]
+                        *intersection, other_reaction_type=j_attrs["reaction_type"]
                     )
                 )
 
@@ -1139,9 +1177,10 @@ def get_geometry_correspondents(
 ) -> dict[Annotation, dict]:
     """
     Returns a copy of 'tagged_annotations' with a 'correspondents' field added to that
-    attributes dictionary of each Annotation key.
+    attribute's dictionary of each Annotation key.
     """
     annots_by_page = annotations_by_page(tagged_annotations)
+
     descending_pages = sorted(annots_by_page.keys(), reverse=True)
     last_page = descending_pages[-1]
     corresponding_annotations = tagged_annotations.copy()
@@ -1152,18 +1191,25 @@ def get_geometry_correspondents(
             next_page = page - 1
             annots_here = annots_by_page[page]
             annots_below = annots_by_page[next_page]
-            correspondents_above = {
-                j_attrs["tag"]: [] for j_attrs in annots_below.values()
-            }
-            correspondents_below = {}
+            # correspondents_above = {
+            #     j_attrs["tag"]: [] for j_attrs in annots_below.values()
+            # }
+            # correspondents_above = {}
+            # correspondents_below = {}
 
             for i_annot, i_attrs in annots_here.items():
+                corresponding_annotations[i_annot].setdefault(
+                    "correspondents_below", []
+                )
                 i_page = i_annot.page
                 i_tag = i_attrs["tag"]
                 i_geom = i_attrs["geometry"]
                 i_rank = i_attrs["rank"]
                 i_rxn_type = i_attrs.get("reaction_type", "point")
                 for j_annot, j_attrs in annots_below.items():
+                    corresponding_annotations[j_annot].setdefault(
+                        "correspondents_above", []
+                    )
                     j_attrs = annots_below[j_annot]
                     j_page = j_annot.page
                     j_geom = j_attrs["geometry"]
@@ -1176,40 +1222,42 @@ def get_geometry_correspondents(
                     ):
                         continue  # No correspondence between lines
                     correspondence_ratio = geom_ops.check_corresponds(i_geom, j_geom)
-                    correspondents_below.setdefault(i_tag, [])
                     if (
                         correspondence_ratio and i_rank >= j_rank
                     ):  # Same rank allowed to transfer in correspondents (e.g. column to column)
-                        correspondents_below[i_tag].append(
-                            Correspondent(
-                                correspondence_ratio,
-                                j_geom,
-                                j_tag,
-                                other_rank=j_rank,
-                                other_reaction_type=j_rxn_type,
-                            )
+                        corr_below = Correspondent(
+                            correspondence_ratio,
+                            j_geom,
+                            j_tag,
+                            other_rank=j_rank,
+                            other_reaction_type=j_rxn_type,
                         )
-                        correspondents_above[j_tag].append(
-                            Correspondent(
-                                correspondence_ratio,
-                                i_geom,
-                                i_attrs["tag"],
-                                other_rank=i_rank,
-                                other_reaction_type=i_rxn_type,
-                            )
-                        )
-                        corresponding_annotations[j_annot].setdefault(
-                            "correspondents_above", []
-                        )
-                        corresponding_annotations[i_annot].setdefault(
-                            "correspondents_below", []
-                        )
-                        corresponding_annotations[j_annot]["correspondents_above"] = (
-                            correspondents_above[j_tag]
-                        )
-                        corresponding_annotations[i_annot]["correspondents_below"] = (
-                            correspondents_below[i_tag]
+                        corr_above = Correspondent(
+                            correspondence_ratio,
+                            i_geom,
+                            i_attrs["tag"],
+                            other_rank=i_rank,
+                            other_reaction_type=i_rxn_type,
                         )
+                        if (
+                            corr_below
+                            not in corresponding_annotations[i_annot][
+                                "correspondents_below"
+                            ]
+                        ):
+                            corresponding_annotations[i_annot][
+                                "correspondents_below"
+                            ].append(corr_below)
+                        if (
+                            corr_above
+                            not in corresponding_annotations[j_annot][
+                                "correspondents_above"
+                            ]
+                        ):
+                            corresponding_annotations[j_annot][
+                                "correspondents_above"
+                            ].append(corr_above)
+
                     else:
                         # Populate empty fields for annotations with no correspondents
                         corresponding_annotations[i_annot].setdefault(
@@ -1219,6 +1267,7 @@ def get_geometry_correspondents(
                             "correspondents_above", []
                         )
         else:
+            correspondents_above = {}
             annots_last = annots_by_page[page]
             if len(descending_pages) == 1:
                 correspondents_above = (
@@ -1235,6 +1284,94 @@ def get_geometry_correspondents(
     return corresponding_annotations
 
 
+def trim_cantilevers(
+    element: Element, rel_tol: float = 2e-2, abs_tol: Optional[float] = None
+):
+    """
+    Mutates the geometry in node elements so that any cantilevers which are
+    below self.cantilever_rel_tol or self.cantilever_abs_tol are removed from
+    the geometry and the geometry spans exactly from support to support.
+    """
+    new_element = deepcopy(element)
+    geometry = element.geometry
+    if geometry.geom_type == "LineString" and not element.extent_polygon:
+        orig_support_geoms = [ib.other_geometry for ib in element.intersections_below]
+        support_geoms = geom_ops.clean_polygon_supports(orig_support_geoms, geometry)
+        support_geoms = geom_ops.sort_supports(geometry, support_geoms)
+        ordered_geom = LineString(
+            geom_ops.order_nodes_positive(
+                [Point(geometry.coords[0]), Point(geometry.coords[-1])]
+            )
+        )
+        cantilevers = geom_ops.get_cantilever_segments(
+            ordered_geom, support_geoms, rel_tol=rel_tol, abs_tol=abs_tol
+        )
+        start_point, end_point = Point(ordered_geom.coords[0]), Point(
+            ordered_geom.coords[-1]
+        )
+        if (cantilevers["A"] == 0.0) and (cantilevers["A"] != cantilevers["A_orig"]):
+            start_point = cantilevers["A_intersection"]
+        if (cantilevers["B"] == 0.0) and (cantilevers["B"] != cantilevers["B_orig"]):
+            end_point = cantilevers["B_intersection"]
+        new_geometry = LineString([start_point, end_point])  # type: ignore
+        new_element.geometry = new_geometry
+        intersection_checks = [
+            new_geometry.intersects(support_geom) for support_geom in support_geoms
+        ]
+        new_intersections_below = [
+            Intersection(
+                intersecting_region=new_geometry.intersection(ib.other_geometry),
+                other_tag=ib.other_tag,
+                other_geometry=ib.other_geometry,
+                other_reaction_type=ib.other_reaction_type,
+                other_extents=ib.other_extents,
+                other_index=ib.other_index,
+            )
+            for ib in element.intersections_below
+        ]
+        new_element.intersections_below = new_intersections_below
+    return new_element
+
+
+def filter_correspondents(
+    tagged_annotations: dict[Annotation, dict],
+) -> dict[Annotation, dict]:
+    """
+    Applies the filter from prioritize_correspondents and back-propagates
+    the filtered correspodents to the correspondents_above:
+    """
+    copied_annotations = tagged_annotations.copy()
+    correspondents_above = {}
+    for annot, annot_attrs in tagged_annotations.items():
+        element_family = annot_attrs["tag"][0]
+        filtered_below = prioritize_correspondents(
+            annot_attrs["correspondents_below"], element_family
+        )
+        copied_annotations[annot]["correspondents_below"] = filtered_below
+        for corr in filtered_below:
+            corr: Correspondent
+            overlap_ratio = corr.overlap_ratio
+            overlap_area = overlap_ratio * annot_attrs["geometry"].area
+            above_overlap_ratio = overlap_area / corr.other_geometry.area
+            above_tag = corr.other_tag
+            corr_above = Correspondent(
+                overlap_ratio=above_overlap_ratio,
+                other_geometry=annot_attrs["geometry"],
+                other_tag=annot_attrs["tag"],
+                other_rank=annot_attrs["rank"],
+                other_reaction_type=annot_attrs["reaction_type"],
+            )
+            correspondents_above.setdefault(above_tag, [])
+            correspondents_above[above_tag].append(corr_above)
+
+    for copied_annot, copied_annot_attrs in copied_annotations.items():
+        copied_annot_attrs["correspondents_above"] = correspondents_above.get(
+            copied_annot_attrs["tag"], []
+        )
+
+    return copied_annotations
+
+
 def annotations_by_page(
     annots: dict[Annotation, dict], ascending=False
 ) -> dict[int, dict[Annotation, dict]]:
