@@ -5,8 +5,9 @@ from decimal import Decimal
 import pathlib
 import networkx as nx
 import hashlib
+import json
 
-from papermodels.datatypes.element import Element, LoadedElement
+from papermodels.datatypes.element import Element, LoadedElement, trim_cantilevers
 from shapely import Point, LineString, Polygon
 from ..geometry import geom_ops as geom
 from ..datatypes.element import (
@@ -21,9 +22,11 @@ from ..paper.annotations import (
     filter_annotations,
     tag_parsed_annotations,
     assign_page_id_to_annotations,
+    annotation_to_shapely,
 )
 from ..paper.plot import plot_annotations
 from ..paper import pdf
+from ..paper import dxf
 from ..datatypes.exceptions import AnnotationError
 from rich.progress import track
 from rich import print
@@ -40,7 +43,12 @@ class GeometryGraph(nx.DiGraph):
     The node_hash is how changes to the graph nodes can be tracked.
     """
 
-    def __init__(self, do_not_process: bool = False):
+    def __init__(
+        self,
+        do_not_process: bool = False,
+        cantilever_rel_tol: float = 2e-2,
+        cantilever_abs_tol: Optional[float] = None,
+    ):
         super().__init__()
         self.do_not_process = do_not_process
         self.node_hash = None
@@ -49,24 +57,39 @@ class GeometryGraph(nx.DiGraph):
         self.raw_annotations = None
         self.legend_entries = None
         self.pdf_path = None
+        self.cantilever_rel_tol: float = cantilever_rel_tol
+        self.cantilever_abs_tol: Optional[float] = cantilever_abs_tol
 
     @property
     def collector_elements(self):
-        return [node for node in self.nodes if not list(self.predecessors(node))]
+        return [
+            node_name
+            for node_name in self.nodes
+            if not list(self.predecessors(node_name))
+            if self.nodes[node_name]["element"].rank == 0
+        ]
 
     @property
     def transfer_elements(self):
-        return [node for node in self.nodes if list(self.predecessors(node))]
+        return [
+            node_name for node_name in self.nodes if list(self.predecessors(node_name))
+        ]
 
     @classmethod
     def from_elements(
-        cls, elements: list[Element], do_not_process: bool = False
+        cls,
+        elements: list[Element],
+        do_not_process: bool = False,
+        cantilever_rel_tol: float = 2e-2,
+        cantilever_abs_tol: Optional[float] = None,
     ) -> GeometryGraph:
         """
         Returns a GeometryGraph (networkx.DiGraph) based upon the intersections and correspondents
         of the 'elements'.
         """
-        g = cls()
+        g = cls(
+            cantilever_rel_tol=cantilever_rel_tol, cantilever_abs_tol=cantilever_abs_tol
+        )
         elements_copy = deepcopy(elements)
         for element in elements_copy:
             hash = hashlib.sha256(str(element).encode()).hexdigest()
@@ -106,12 +129,26 @@ class GeometryGraph(nx.DiGraph):
         if do_not_process:
             return g
 
+        g.trim_cantilevers()
         g.remove_excess_correspondent_load_paths()
         g.add_intersection_indexes_below()
         g.add_intersection_indexes_above()
 
         return g
 
+    def trim_cantilevers(self):
+        """
+        Trims cantilevers if they are within the tolerance
+        """
+        node_names = self.transfer_elements
+        for node_name in node_names:
+            node = self.nodes[node_name]
+            element = node["element"]
+            new_element = trim_cantilevers(
+                element, self.cantilever_rel_tol, self.cantilever_abs_tol
+            )
+            node["element"] = new_element
+
     def remove_excess_correspondent_load_paths(self):
         """
         Removes edges from the graph for the following conditions:
@@ -271,6 +308,8 @@ class GeometryGraph(nx.DiGraph):
             dependents = list(self.successors(node))
             dependent_intersections = get_dependent_intersections(element, dependents)
             dependent_correspondents = get_dependent_correspondents(element, dependents)
+            if not dependent_intersections and not dependent_correspondents:
+                continue
             if element.geometry.geom_type == "Polygon":  # node geometry is polygon
                 updated_intersections_below = []
                 all_extents = {}
@@ -280,7 +319,7 @@ class GeometryGraph(nx.DiGraph):
                     extents = all_extents.get(intersection.other_tag)
                     new_intersection = Intersection(
                         intersection.intersecting_region,
-                        intersection.other_geometry,
+                        self.nodes[intersection.other_tag]["element"].geometry,
                         intersection.other_tag,
                         0,
                         intersection.other_reaction_type,
@@ -323,6 +362,7 @@ class GeometryGraph(nx.DiGraph):
                             sub_local_coords = get_local_coords(
                                 start_coord, sub_dependent_intersections
                             )
+                            sub_id = subelem.tag
                             subextents = subelem.get_collector_extents()
                             sub_sorted_below_ints = sorted(
                                 sub_local_coords, key=lambda x: x[0]
@@ -363,7 +403,7 @@ class GeometryGraph(nx.DiGraph):
                             local_index = other_tags_below.index(other_tag)
                             new_intersection = Intersection(
                                 intersection.intersecting_region,
-                                intersection.other_geometry,
+                                self.nodes[intersection.other_tag]["element"].geometry,
                                 intersection.other_tag,
                                 local_index,
                                 other_reaction_type=intersection.other_reaction_type,
@@ -381,7 +421,7 @@ class GeometryGraph(nx.DiGraph):
                         local_index = other_tags_below.index(other_tag)
                         new_intersection = Intersection(
                             intersection.intersecting_region,
-                            intersection.other_geometry,
+                            self.nodes[intersection.other_tag]["element"].geometry,
                             intersection.other_tag,
                             local_index,
                             other_reaction_type=intersection.other_reaction_type,
@@ -412,6 +452,8 @@ class GeometryGraph(nx.DiGraph):
                 for correspondent in element.correspondents_above
                 if correspondent.other_tag in predecessors
             ]
+            if not predecessor_intersections and not predecessor_correspondents:
+                continue
             indexed_intersections_above = []
             for intersection in predecessor_intersections:
                 other_tag = intersection.other_tag
@@ -434,7 +476,7 @@ class GeometryGraph(nx.DiGraph):
                     other_extents = above_intersections_below[element_tag][1]
                     new_intersection = Intersection(
                         intersection.intersecting_region,
-                        intersection.other_geometry,
+                        element.geometry,
                         intersection.other_tag,
                         local_index,
                         element_above.reaction_type,
@@ -562,6 +604,147 @@ class GeometryGraph(nx.DiGraph):
         self.add_intersection_indexes_below()
         self.add_intersection_indexes_above()
 
+    @classmethod
+    def from_dxf_file(
+        cls,
+        dxf_filepath: pathlib.Path | str,
+        legend_table: dict | pathlib.Path | str,
+        page_idx: int = 0,
+        scale: Decimal = Decimal(1.0),
+        polygonize_layers: Optional[list[str]] = None,
+        debug: bool = False,
+        progress: bool = False,
+        do_not_process: bool = False,
+        show_skipped: bool = False,
+    ):
+        """
+        Returns a GeometryGraph built from the geometric entities (LINE, LWPOLYLINE, INSERT)
+        contained within 'dxf_filepath'.
+
+
+        'legend_table': A dict (or a path to a JSON file) that maps layer names to
+            annotation text properties
+        'scale': An optional scale to be applied to the annotations. If not provided,
+            the units of the annotations will be in PDF points where 1 point == 1 /72 inch
+        'debug':  When True, will provide verbose documentation of the annotation parsing
+            process to assist in reviewing errors and geometry inconsistencies.
+        'progress': When True, a progress bar will be displayed
+        'do_not_process': Reads the file and adds annotations to the graph but does not
+            process the connectivity. Useful for debugging and plotting prior to processing.
+        'show_skipped': Shows the skipped annotations that occured during pdf.load_pdf_annotations
+        """
+        if isinstance(legend_table, (str, pathlib.Path)):
+            legend_table_path = pathlib.Path(legend_table)
+            with open(legend_table_path, "r") as file:
+                legend_table = json.load(file)
+        annotations = dxf.load_dxf_annotations(
+            dxf_filepath, page_idx, polygonize_layers=polygonize_layers
+        )
+        scaled_annotations = scale_annotations(
+            annotations, scale=scale, paper_origin=(0, 0)
+        )
+
+        # parsed_annotations = parse_annotations(
+        #     scaled_annots_in_page, legend_entries, legend_identifier
+        # )
+
+        load_entries = {}
+        trib_area_entries = {}
+        structural_element_entries = {}
+        parsed_annotations = {}
+        raw_annotations = {}
+        for idx, scaled_annot in enumerate(scaled_annotations):
+            raw_annot = annotations[idx]
+            layer_name = scaled_annot.text
+            annot_attrs = legend_table.get(layer_name)
+            if annot_attrs is None:
+                continue
+            annot_attrs = {k.lower(): v for k, v in annot_attrs.items()}
+
+            annot_attrs.update({"extent_polygon": None})
+            existing_annot_tag = annot_attrs.get("tag", None)
+            annot_geom = annotation_to_shapely(scaled_annot)
+            annot_attrs["geometry"] = annot_geom
+            annot_attrs["page_label"] = scaled_annot.page
+            annot_attrs["tag"] = existing_annot_tag
+
+            if "extent" in annot_attrs["type"]:
+                parsed_annotations.update({scaled_annot: annot_attrs})
+            else:
+                annot_attrs.setdefault("reaction_type", "point")
+                annot_attrs["reaction_type"] = annot_attrs["reaction_type"].lower()
+                if (
+                    annot_geom.geom_type == "Polygon"
+                    and annot_attrs["reaction_type"] == "linear"
+                ):
+                    annot_attrs["length"] = geom.get_rectangle_centerline(
+                        annot_geom
+                    ).length
+                parsed_annotations.update({scaled_annot: annot_attrs})
+
+            parsed_annotations[scaled_annot] = annot_attrs
+            raw_annotations[raw_annot] = annot_attrs
+
+            if "occupancy" in annot_attrs:
+                load_entries.update({scaled_annot: annot_attrs})
+            elif "type" in annot_attrs and "hole" in annot_attrs["type"].lower():
+                load_entries.update({scaled_annot: annot_attrs})
+            elif "type" in annot_attrs and "trib area" in annot_attrs["type"].lower():
+                trib_area_entries.update({scaled_annot: annot_attrs})
+            elif "type" not in annot_attrs:
+                continue
+            else:
+                structural_element_entries.update({scaled_annot: annot_attrs})
+
+        elements = Element.from_parsed_annotations(
+            structural_element_entries, trib_area_entries
+        )
+        graph = cls.from_elements(elements, do_not_process=do_not_process)
+        graph.parsed_annotations = tag_parsed_annotations(parsed_annotations)
+        graph.raw_annotations = tag_parsed_annotations(raw_annotations)
+        graph.legend_entries = {}
+        graph.loading_geometries = parsed_annotations_to_loading_geometry(load_entries)
+        return graph
+
+    def parse_annotations():
+        parsed_annotations = {}
+        for legend_item in legend:
+            legend_properties = {
+                prop: getattr(legend_item, prop) for prop in properties_to_match
+            }
+            matching_annots = filter_annotations(annots, legend_properties)
+            annot_attributes = parse_legend(legend_item.text, legend_identifier)
+            for annot in matching_annots:
+                if annot in legend:
+                    continue
+                annot_kwargs = parse_annot_kwargs(annot.text)
+                existing_annot_tag = annot_kwargs.get("tag", None)
+                annot_geom = annotation_to_shapely(annot)
+                annot_attrs = {}
+                annot_attrs["geometry"] = annot_geom
+                annot_attrs["page_label"] = annot.page
+                annot_attrs["tag"] = existing_annot_tag
+                for annot_key, annot_attr in annot_attributes.items():
+                    annot_attrs[annot_key] = str_to_int(
+                        annot_attr.split("<")[0]
+                    )  # .split() to remove trailing HTML tags
+
+                # Run tests for this first
+                # annot_attrs["rank"] = int(annot_attributes["rank"])
+                if "extent" in annot_attrs["type"]:
+                    parsed_annotations.update({annot: annot_attrs})
+                else:
+                    annot_attrs.setdefault("reaction_type", "point")
+                    annot_attrs["reaction_type"] = annot_attrs["reaction_type"].lower()
+                    if (
+                        annot_geom.geom_type == "Polygon"
+                        and annot_attrs["reaction_type"] == "linear"
+                    ):
+                        annot_attrs["length"] = geom_ops.get_rectangle_centerline(
+                            annot_geom
+                        ).length
+                    parsed_annotations.update({annot: annot_attrs | annot_kwargs})
+
     def unassigned_collectors(self) -> list[str]:
         """
         Returns a list of str that represents collector nodes who do not currently
@@ -578,9 +761,11 @@ class GeometryGraph(nx.DiGraph):
     @classmethod
     def from_pdf_file(
         cls,
-        pdf_filepath: pathlib.path | str,
+        pdf_filepath: pathlib.Path | str,
         legend_identifier: str = "legend",
         scale: Optional[Decimal] = None,
+        cantilever_rel_tol: float = 2e-2,
+        cantilever_abs_tol: Optional[float] = None,
         debug: bool = False,
         progress: bool = False,
         do_not_process: bool = False,
@@ -590,7 +775,7 @@ class GeometryGraph(nx.DiGraph):
     ):
         """
         Returns a GeometryGraph built from that annotations in the provided PDF file
-        at 'filepath'.
+        at 'pdf_filepath'.
 
         The provided annotations are parsed into four different categories:
             0. Legend entries - All legend entries must contain the 'legend_identifier'
@@ -622,7 +807,12 @@ class GeometryGraph(nx.DiGraph):
         """
         annotations = pdf.load_pdf_annotations(pdf_filepath, show_skipped)
         graph = cls.from_annotations(
-            annotations, legend_identifier, scale=scale, do_not_process=do_not_process
+            annotations,
+            legend_identifier,
+            scale=scale,
+            do_not_process=do_not_process,
+            cantilever_rel_tol=cantilever_rel_tol,
+            cantilever_abs_tol=cantilever_abs_tol,
         )
         graph.pdf_path = pathlib.Path(pdf_filepath).resolve()
         return graph
@@ -633,6 +823,8 @@ class GeometryGraph(nx.DiGraph):
         annotations: list[Annotation],
         legend_identifier: str = "legend",
         scale: Optional[Decimal] = None,
+        cantilever_rel_tol: float = 2e-2,
+        cantilever_abs_tol: Optional[float] = None,
         # area_load_properties: Optional[dict] = None,
         # trib_area_properties: Optional[dict] = None,
         debug: bool = False,
@@ -728,18 +920,75 @@ class GeometryGraph(nx.DiGraph):
                 structural_element_entries, extent_entries
             )
 
+        if not structural_element_entries and not legend_entries:
+            raise ValueError(
+                "No structural element entities were found.\n"
+                "Did you forget to create some legend entries?"
+            )
+
+        elif not structural_element_entries:
+            raise ValueError(
+                "No structural element entities were found.\n"
+                "Do the annotation properties match the legend entry properties?"
+            )
+
         elements = Element.from_parsed_annotations(
             structural_element_entries, trib_area_entries
         )
-        graph = cls.from_elements(elements, do_not_process=do_not_process)
+        graph = cls.from_elements(
+            elements,
+            cantilever_rel_tol=cantilever_rel_tol,
+            cantilever_abs_tol=cantilever_abs_tol,
+            do_not_process=do_not_process,
+        )
         graph.parsed_annotations = tag_parsed_annotations(parsed_annotations_acc)
         graph.raw_annotations = tag_parsed_annotations(raw_annotations_acc)
         graph.legend_entries = legend_entries
         graph.loading_geometries = parsed_annotations_to_loading_geometry(load_entries)
         return graph
 
-    def plot_connectivity(self):
-        return nx.draw_spectral(self, with_labels=True)
+    def plot_connectivity(self, filepath: Optional[pathlib.Path | str] = None) -> None:
+        """
+        Using GraphViz, this function plots the connectivity, a representation of
+        a load path, between members where each member is represented as a node in
+        the graph and each connection is represented by a directional edge (arrow).
+
+        If 'filepath' is provided, then the resulting SVG image of the graph will
+        be saved to disk.
+
+        If 'filepath' is None, then display of the SVG will be attempted through
+        IPython.display.
+
+        If the designer is not within a Jupyter-like environment, then the SVG
+        string will be returned.
+
+        Requires GraphViz to be independently installed. Installation instructions
+        here: https://graphviz.org/download/
+        """
+        try:
+            plotting = nx.drawing.nx_agraph.to_agraph(self)
+        except:
+            raise ImportError(
+                "The GraphViz application is missing. Install at https://graphviz.org/download/"
+            )
+        plotting.layout(prog="dot")
+        svg_string = plotting.draw(format="svg")
+        if filepath:
+            with open(filepath, "wb") as file:
+                file.write(svg_string)
+                return
+        else:
+            try:
+                from IPython.display import SVG, display
+            except ImportError:
+                print(svg_string)
+                return
+            try:
+                get_ipython
+            except NameError:
+                display(svg_string)
+                return
+            display(SVG(svg_string))
 
     def plot_annotations(
         self, page_idx: int, figsize: tuple[float, float] = (8, 8), dpi: int = 150
